#!/usr/bin/perl -w

use strict;
use LWP::Simple;
use Digest::MD5 qw(md5_hex);

###
# configuration
###
my $siteFile = "/home/jeff2619/karamba/scripts/sites";
my $sigFile = "/home/jeff2619/karamba/scripts/sig.dat";

######
#
# originally written by Keith Jeffery
# jeff2619@uidaho.edu
# April 13, 2003
#
# reads $siteFile and checks the sites listed there to see if they've been
# updated since last check.
#
# $siteFile is in the following format:
# <url> <name>
#
# $sigFile is in the following format:
# <md5> <name>
#
# you shouldn't ever have to hand-modify the $sigFile though
#
# these formats allow for spaces in names, this format also assumes that there
# are no spaces in the url (bad mojo, what kind of idiot puts spaces in their
# urls anyway?)
#
# Output of this program is in the following format:
# 1: Modified Site Name
# 2: URL
# 3: "Update"
# 4: MD5 Name
# 5: Modified Site Name
# 6: ...
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
######

my %sites;
my %sigs;

# getting sites to check from file "sites"
open(SITES, "<$siteFile") or die("Cannot open file \"$siteFile\"\n\n\n");

while (<SITES>) {
  my $url;
  my $name;

  chomp;

  # URL and name are split up by spaces
  my @site = split(/ /);
  $url = shift(@site);

  # but site names may have spaces
  $name = join(' ', @site);
  $sites{$name} = $url;
}

close(SITES);

if (!open(SIGS, "<$sigFile")) {
  my $ret;
  
  warn("Cannot open file \"$sigFile\"\n");
  print("Trying to create \"$sigFile\"\n");
  $ret = system("touch", "$sigFile");
  if ($ret) {
    die("Cannot create \"$sigFile\"");
  } else {
    print("Success!\n");
  }
} else {
  while (<SIGS>) {
    my $name;
    my $sig;

    chomp;

    my @sigArray = split(/ /);
    $sig = shift(@sigArray);
    $name = join(' ', @sigArray);
    $sigs{$name} = $sig;
  }
}
  
close(SIGS);

my $key;
my %newItems;
my $updated;
my %updatedSigs;

$updated = compareKeys(\%sites, \%sigs, \%updatedSigs, \%newItems);

if ($updated) {
  foreach $key (keys %newItems) {
    print "$key\n"; # site name
    print "$newItems{$key}\n"; # site URL
    print "Update\n"; # hack to get the behavior I want in Karamba
    print "$updatedSigs{$key} $key\n"; # new MD5 and name
  }
}


###       ###
# Functions #
###       ###


sub compareKeys {
  my $sites = shift;
  my $sigs = shift;
  my $updatedSigs = shift;
  my $newItems = shift;

  my $updated = 0;

  my $key;
  my $url;
  my $content;
  my $md5;

  foreach $key (keys %sites) {
    # open up each page
    $url = $sites->{$key};
    $content = get $url; # this just dumps the entire page into $content
    if (!defined($content)) {
      warn "Could not open $url\n";
      next;
    }

    # this is for the ILM sites, which timestamp their HTML
    # with 03:49:30 GMT
    $content =~ s/[0-9]+\:[0-9]+\:[0-9]+ GMT//;

    $md5 = md5_hex($content); # calculate the md5 hex!

    if (!exists($sigs->{$key}) || $sigs->{$key} ne $md5) {
      # the site has changed or it hasn't been put into the sig file before
      $updated++;
      $newItems->{$key} = $url;
    }
    
    # the new sigs hash to output to file later
    $updatedSigs->{$key} = $md5;
  }
  return $updated;
} # end compareKeys()
